@@ -249,8 +249,10 @@ params = [
                     ['D', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED],
                     ['E', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED]
                 ],
-                'weight': [1, 1, 1, 1, 1, 1, 5, 5]
+                'weight': [1, 1, 1, 1, 1, 1, 5, 5],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
             },
+            'split_fields': ['precinct'],
             'exhaust_on_duplicate_candidate_marks': True,
             'exhaust_on_overvote_marks': True,
             'exhaust_on_repeated_skipped_marks': True
@@ -363,8 +365,10 @@ params = [
                     ['C', 'D', BallotMarks.WRITEIN, 'B'],
                     ['C', 'D', 'D', 'B'],
                     ['D', 'B', BallotMarks.WRITEIN, 'D']
-                ]
-            }
+                ],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
+            },
+            'split_fields': ['precinct']
         },
         'expected': {
             'stat': 2
@@ -424,8 +428,10 @@ params = [
                     ['C', 'D', BallotMarks.WRITEIN, 'B'],
                     ['C', 'D', 'D', 'B'],
                     ['D', 'B', BallotMarks.WRITEIN, 'D']
-                ]
-            }
+                ],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
+            },
+            'split_fields': ['precinct']
         },
         'expected': {
             'stat': 4
@@ -485,8 +491,10 @@ params = [
                     ['C', 'D', BallotMarks.WRITEIN, 'B'],
                     ['C', 'D', 'D', 'B'],
                     ['D', 'B', BallotMarks.WRITEIN, 'D']
-                ]
-            }
+                ],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
+            },
+            'split_fields': ['precinct']
         },
         'expected': {
             'stat': 25
@@ -546,8 +554,10 @@ params = [
                     ['C', 'D', BallotMarks.WRITEIN, 'B'],
                     ['C', 'D', 'D', 'B'],
                     ['D', 'B', BallotMarks.WRITEIN, 'D']
-                ]
-            }
+                ],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
+            },
+            'split_fields': ['precinct']
         },
         'expected': {
             'stat': round(100 * 4/7, 3)
@@ -607,8 +617,10 @@ params = [
                     ['C', 'D', BallotMarks.WRITEIN, 'B'],
                     ['C', 'D', 'D', 'B'],
                     ['D', 'B', BallotMarks.WRITEIN, 'D']
-                ]
-            }
+                ],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
+            },
+            'split_fields': ['precinct']
         },
         'expected': {
             'stat': 2
@@ -668,11 +680,13 @@ params = [
                     ['C', 'D', BallotMarks.WRITEIN, 'B'],
                     ['C', 'D', 'D', 'B'],
                     ['D', 'B', BallotMarks.WRITEIN, 'D']
-                ]
-            }
+                ],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
+            },
+            'split_fields': ['precinct']
         },
         'expected': {
-            'stat': True
+            'stat': False
         }
     })
 ]
@@ -729,8 +743,10 @@ params = [
                     ['C', 'D', BallotMarks.WRITEIN, 'B'],
                     ['C', 'D', 'D', 'B'],
                     ['D', 'B', BallotMarks.WRITEIN, 'D']
-                ]
-            }
+                ],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
+            },
+            'split_fields': ['precinct']
         },
         'expected': {
             'stat': True
@@ -790,8 +806,10 @@ params = [
                     ['C', 'D', BallotMarks.WRITEIN, 'B'],
                     ['C', 'D', 'D', 'B'],
                     ['D', 'B', BallotMarks.WRITEIN, 'D']
-                ]
-            }
+                ],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
+            },
+            'split_fields': ['precinct']
         },
         'expected': {
             'stat': 5
@@ -851,8 +869,10 @@ params = [
                     ['C', 'D', BallotMarks.WRITEIN, 'B'],
                     ['C', 'D', 'D', 'B'],
                     ['D', 'B', BallotMarks.WRITEIN, 'D']
-                ]
-            }
+                ],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
+            },
+            'split_fields': ['precinct']
         },
         'expected': {
             'stat': 50
@@ -912,8 +932,10 @@ params = [
                     ['C', 'D', BallotMarks.WRITEIN, 'B'],
                     ['C', 'D', 'D', 'B'],
                     ['D', 'B', BallotMarks.WRITEIN, 'D']
-                ]
-            }
+                ],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
+            },
+            'split_fields': ['precinct']
         },
         'expected': {
             'stat': None
@@ -923,9 +945,9 @@ params = [
 
 
 @pytest.mark.parametrize("param", params)
-def test_win_threshold(param):
+def test_static_win_threshold(param):
     rcv = SingleWinner(**param['input'])
-    assert rcv.stats()[0]['win_threshold'].item() == param['expected']['stat']
+    assert rcv.stats()[0]['static_win_threshold'].item() == param['expected']['stat']
 
 
 params = [
@@ -973,8 +995,10 @@ params = [
                     ['C', 'D', BallotMarks.WRITEIN, 'B'],
                     ['C', 'D', 'D', 'B'],
                     ['D', 'B', BallotMarks.WRITEIN, 'D']
-                ]
-            }
+                ],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
+            },
+            'split_fields': ['precinct']
         },
         'expected': {
             'stat': 1
@@ -984,9 +1008,9 @@ params = [
 
 
 @pytest.mark.parametrize("param", params)
-def test_number_of_winners(param):
+def test_number_of_tabulation_winners(param):
     rcv = SingleWinner(**param['input'])
-    assert rcv.stats()[0]['number_of_winners'].item() == param['expected']['stat']
+    assert rcv.stats()[0]['number_of_tabulation_winners'].item() == param['expected']['stat']
 
 
 params = [
@@ -1019,7 +1043,7 @@ params = [
             }
         },
         'expected': {
-            'stat': BallotMarks.WRITEIN.title()
+            'stat': BallotMarks.WRITEIN
         }
     }),
     ({
@@ -1034,11 +1058,13 @@ params = [
                     ['C', 'D', BallotMarks.WRITEIN, 'B'],
                     ['C', 'D', 'D', 'B'],
                     ['D', 'B', BallotMarks.WRITEIN, 'D']
-                ]
-            }
+                ],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
+            },
+            'split_fields': ['precinct']
         },
         'expected': {
-            'stat': BallotMarks.WRITEIN.title()
+            'stat': BallotMarks.WRITEIN
         }
     })
 ]
@@ -1095,8 +1121,10 @@ params = [
                     ['C', 'D', BallotMarks.WRITEIN, 'B'],
                     ['C', 'D', 'D', 'B'],
                     ['D', 'B', BallotMarks.WRITEIN, 'D']
-                ]
-            }
+                ],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
+            },
+            'split_fields': ['precinct']
         },
         'expected': {
             'stat': 100 * 5/8
@@ -1176,8 +1204,10 @@ params = [
                     ['C', 'D', 'D', 'B'],
                     ['D', 'B', BallotMarks.WRITEIN, 'D']
                 ],
-                'weight': [1, 1, 1, 2, 2, 1, 1, 1]
-            }
+                'weight': [1, 1, 1, 2, 2, 1, 1, 1],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
+            },
+            'split_fields': ['precinct']
         },
         'expected': {
             'stat': 10
@@ -1257,8 +1287,10 @@ params = [
                     ['C', 'D', 'D', 'B'],
                     ['D', 'B', BallotMarks.WRITEIN, 'D']
                 ],
-                'weight': [1, 1, 1, 2, 2, 1, 1, 1]
-            }
+                'weight': [1, 1, 1, 2, 2, 1, 1, 1],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
+            },
+            'split_fields': ['precinct']
         },
         'expected': {
             'stat': 9
@@ -1340,8 +1372,10 @@ params = [
                     ['D', 'B', BallotMarks.WRITEIN, 'D'],
                     [BallotMarks.OVERVOTE, BallotMarks.OVERVOTE, BallotMarks.OVERVOTE, BallotMarks.OVERVOTE]
                 ],
-                'weight': [1, 1, 1, 2, 2, 1, 1, 1, 5]
-            }
+                'weight': [1, 1, 1, 2, 2, 1, 1, 1, 5],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3, 4]
+            },
+            'split_fields': ['precinct']
         },
         'expected': {
             'stat': 5
@@ -1422,8 +1456,10 @@ params = [
                     ['D', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED],
                     ['E', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED]
                 ],
-                'weight': [1, 1, 1, 1, 1, 1, 5, 5]
+                'weight': [1, 1, 1, 1, 1, 1, 5, 5],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
             },
+            'split_fields': ['precinct'],
             'exhaust_on_duplicate_candidate_marks': True,
             'exhaust_on_overvote_marks': True,
             'exhaust_on_repeated_skipped_marks': True
@@ -1455,8 +1491,10 @@ params = [
                     ['D', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED],
                     ['E', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED]
                 ],
-                'weight': [1, 1, 1, 1, 1, 1, 5, 5]
+                'weight': [1, 1, 1, 1, 1, 1, 5, 5],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
             },
+            'split_fields': ['precinct'],
             'exhaust_on_duplicate_candidate_marks': True,
             'exhaust_on_overvote_marks': True,
             'exhaust_on_repeated_skipped_marks': True
@@ -1488,8 +1526,10 @@ params = [
                     ['D', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED],
                     ['E', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED]
                 ],
-                'weight': [1, 1, 1, 1, 1, 1, 5, 5]
+                'weight': [1, 1, 1, 1, 1, 1, 5, 5],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
             },
+            'split_fields': ['precinct'],
             'exhaust_on_duplicate_candidate_marks': True,
             'exhaust_on_overvote_marks': True,
             'exhaust_on_repeated_skipped_marks': True
@@ -1521,8 +1561,10 @@ params = [
                     ['D', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED],
                     ['E', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED]
                 ],
-                'weight': [1, 1, 1, 1, 1, 1, 5, 5]
+                'weight': [1, 1, 1, 1, 1, 1, 5, 5],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
             },
+            'split_fields': ['precinct'],
             'exhaust_on_duplicate_candidate_marks': True,
             'exhaust_on_overvote_marks': True,
             'exhaust_on_repeated_skipped_marks': True
@@ -1554,8 +1596,10 @@ params = [
                     ['D', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED],
                     ['E', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED]
                 ],
-                'weight': [1, 1, 1, 1, 1, 1, 5, 5]
+                'weight': [1, 1, 1, 1, 1, 1, 5, 5],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
             },
+            'split_fields': ['precinct'],
             'exhaust_on_duplicate_candidate_marks': True,
             'exhaust_on_overvote_marks': True,
             'exhaust_on_repeated_skipped_marks': True
@@ -1587,8 +1631,80 @@ params = [
                     ['D', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED],
                     ['E', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED]
                 ],
-                'weight': [1, 1, 1, 1, 1, 1, 5, 5]
+                'weight': [1, 1, 1, 1, 1, 1, 5, 5],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
             },
+            'split_fields': ['precinct'],
+            'exhaust_on_duplicate_candidate_marks': True,
+            'exhaust_on_overvote_marks': True,
+            'exhaust_on_repeated_skipped_marks': True
+        },
+        'expected': {
+            'stat': 0
+        }
+    })
+]
+
+
+@pytest.mark.parametrize("param", params)
+def test_total_posttally_exhausted_by_rank_limit_fully_ranked(param):
+    rcv = SingleWinner(**param['input'])
+    assert rcv.stats()[0]['total_posttally_exhausted_by_rank_limit_fully_ranked'].item() == param['expected']['stat']
+
+
+params = [
+    ({
+        'input': {
+            'parsed_cvr': {
+                'ranks': [
+                    ['A', 'C', 'C', 'D'],
+                    [BallotMarks.OVERVOTE, 'A', 'B', 'C'],
+                    ['B', BallotMarks.OVERVOTE, 'A', 'B'],
+                    ['B', 'A', 'A', 'C'],
+                    ['C', BallotMarks.SKIPPED, BallotMarks.SKIPPED, 'A'],
+                    ['C', 'A', 'B', 'E'],
+                    ['D', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED],
+                    ['E', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED]
+                ],
+                'weight': [1, 1, 1, 1, 1, 1, 5, 5],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
+            },
+            'split_fields': ['precinct'],
+            'exhaust_on_duplicate_candidate_marks': True,
+            'exhaust_on_overvote_marks': True,
+            'exhaust_on_repeated_skipped_marks': True
+        },
+        'expected': {
+            'stat': 0
+        }
+    })
+]
+
+
+@pytest.mark.parametrize("param", params)
+def test_total_posttally_exhausted_by_rank_limit_partially_ranked(param):
+    rcv = SingleWinner(**param['input'])
+    assert rcv.stats()[0]['total_posttally_exhausted_by_rank_limit_partially_ranked'].item() == param['expected']['stat']
+
+
+params = [
+    ({
+        'input': {
+            'parsed_cvr': {
+                'ranks': [
+                    ['A', 'C', 'C', 'D'],
+                    [BallotMarks.OVERVOTE, 'A', 'B', 'C'],
+                    ['B', BallotMarks.OVERVOTE, 'A', 'B'],
+                    ['B', 'A', 'A', 'C'],
+                    ['C', BallotMarks.SKIPPED, BallotMarks.SKIPPED, 'A'],
+                    ['C', 'A', 'B', 'E'],
+                    ['D', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED],
+                    ['E', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED]
+                ],
+                'weight': [1, 1, 1, 1, 1, 1, 5, 5],
+                'precinct': [1, 1, 1, 2, 2, 2, 3, 3]
+            },
+            'split_fields': ['precinct'],
             'exhaust_on_duplicate_candidate_marks': True,
             'exhaust_on_overvote_marks': True,
             'exhaust_on_repeated_skipped_marks': True
@@ -1928,6 +2044,76 @@ def test_split_total_posttally_exhausted_by_rank_limit(param):
     assert rcv.stats(add_split_stats=True)[0]['split_total_posttally_exhausted_by_rank_limit'].tolist() == param['expected']['stat']
 
 
+params = [
+    ({
+        'input': {
+            'parsed_cvr': {
+                'ranks': [
+                    ['A', 'C', 'C', 'D'],
+                    [BallotMarks.OVERVOTE, 'A', 'B', 'C'],
+                    ['B', BallotMarks.OVERVOTE, 'A', 'B'],
+                    ['B', 'A', 'A', 'C'],
+                    ['C', BallotMarks.SKIPPED, BallotMarks.SKIPPED, 'A'],
+                    ['C', 'A', 'B', 'E'],
+                    ['D', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED],
+                    ['E', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED]
+                ],
+                'weight': [1, 1, 1, 1, 1, 1, 5, 5],
+                'split': [1, 1, 1, 1, 2, 2, 2, 3]
+            },
+            'split_fields': ['split'],
+            'exhaust_on_duplicate_candidate_marks': True,
+            'exhaust_on_overvote_marks': True,
+            'exhaust_on_repeated_skipped_marks': True
+        },
+        'expected': {
+            'stat': [0, 0, 0]
+        }
+    })
+]
+
+
+@pytest.mark.parametrize("param", params)
+def test_split_total_posttally_exhausted_by_rank_limit_fully_ranked(param):
+    rcv = SingleWinner(**param['input'])
+    assert rcv.stats(add_split_stats=True)[0]['split_total_posttally_exhausted_by_rank_limit_fully_ranked'].tolist() == param['expected']['stat']
+
+
+params = [
+    ({
+        'input': {
+            'parsed_cvr': {
+                'ranks': [
+                    ['A', 'C', 'C', 'D'],
+                    [BallotMarks.OVERVOTE, 'A', 'B', 'C'],
+                    ['B', BallotMarks.OVERVOTE, 'A', 'B'],
+                    ['B', 'A', 'A', 'C'],
+                    ['C', BallotMarks.SKIPPED, BallotMarks.SKIPPED, 'A'],
+                    ['C', 'A', 'B', 'E'],
+                    ['D', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED],
+                    ['E', BallotMarks.SKIPPED, BallotMarks.SKIPPED, BallotMarks.SKIPPED]
+                ],
+                'weight': [1, 1, 1, 1, 1, 1, 5, 5],
+                'split': [1, 1, 1, 1, 2, 2, 2, 3]
+            },
+            'split_fields': ['split'],
+            'exhaust_on_duplicate_candidate_marks': True,
+            'exhaust_on_overvote_marks': True,
+            'exhaust_on_repeated_skipped_marks': True
+        },
+        'expected': {
+            'stat': [0, 0, 0]
+        }
+    })
+]
+
+
+@pytest.mark.parametrize("param", params)
+def test_split_total_posttally_exhausted_by_rank_limit_partially_ranked(param):
+    rcv = SingleWinner(**param['input'])
+    assert rcv.stats(add_split_stats=True)[0]['split_total_posttally_exhausted_by_rank_limit_partially_ranked'].tolist() == param['expected']['stat']
+
+
 params = [
     ({
         'input': {
@@ -1961,5 +2147,3 @@ params = [
 def test_split_total_posttally_exhausted_by_duplicate_rankings(param):
     rcv = SingleWinner(**param['input'])
     assert rcv.stats(add_split_stats=True)[0]['split_total_posttally_exhausted_by_duplicate_rankings'].tolist() == param['expected']['stat']
-
-
